(function() {
  var CRLF, Log, all_subs, argv, clients, host, log, net, next_client_id, port, server, _ref;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  net = require('net');
  argv = require('optimist')["default"]({
    listen: '127.0.0.1:9912',
    loglevel: 'error'
  }).argv;
  Log = require('log');
  log = new Log(argv.loglevel);
  CRLF = '\r\n';
  next_client_id = 0;
  clients = {};
  all_subs = {};
  server = net.createServer(function(client) {
    client.setTimeout(0);
    client.setKeepAlive(true);
    client.setEncoding('utf8');
    client.id = next_client_id++;
    clients[client.id] = client;
    client.subs = {};
    client.on('data', __bind(function(data) {
      var client_id, line, msg, _, _i, _len, _ref, _results;
      try {
        _ref = data.split('\r\n');
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          line = _ref[_i];
          if (!(line.length > 0)) {
            continue;
          }
          msg = JSON.parse(line);
          _results.push((function() {
            var _name, _ref2, _results2;
            if (msg.sub != null) {
              log.debug("client " + client.id + " subscribes to " + msg.sub);
              client.subs[msg.sub] = true;
              return (all_subs[_name = msg.sub] || (all_subs[_name] = {}))[client.id] = true;
            } else if (msg.pub != null) {
              log.debug("client " + client.id + " publishes " + msg.pub.msg + " to " + msg.pub.chan);
              _ref2 = all_subs[msg.pub.chan] || {};
              _results2 = [];
              for (client_id in _ref2) {
                _ = _ref2[client_id];
                log.debug("forwarding message to client " + client_id);
                _results2.push(clients[client_id].write(JSON.stringify(msg) + CRLF));
              }
              return _results2;
            } else {
              throw new Error('unknown msg type');
            }
          })());
        }
        return _results;
      } catch (err) {
        log.error("client " + client.id + ": " + (err.toString()));
        return client.destroy();
      }
    }, this));
    return client.on('close', __bind(function() {
      var chan, _, _ref;
      log.debug("client " + client.id + " closed");
      _ref = client.subs;
      for (chan in _ref) {
        _ = _ref[chan];
        delete all_subs[chan][client.id];
        if (all_subs[chan].length === 0) {
          delete all_subs[chan];
        }
      }
      return delete clients[client.id];
    }, this));
  });
  _ref = argv.listen.split(':'), host = _ref[0], port = _ref[1];
  server.listen(port, host, function() {
    return log.info("simple pubsub server listening on " + host + ":" + port);
  });
  process.on('uncaughtException', function(err) {
    var client, _i, _len, _results;
    log.error("uncaught exception! " + (err.toString()));
    _results = [];
    for (_i = 0, _len = clients.length; _i < _len; _i++) {
      client = clients[_i];
      _results.push(client.write(JSON.stringify({
        msg: {
          msg: err.toString(),
          chan: 'simplepubsub'
        }
      }) + CRLF));
    }
    return _results;
  });
}).call(this);
