(function() {
  var CRLF, EventEmitter, SimplePubsubClient, net;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  net = require('net');
  EventEmitter = require('events').EventEmitter;
  CRLF = '\r\n';
  SimplePubsubClient = (function() {
    __extends(SimplePubsubClient, EventEmitter);
    function SimplePubsubClient(port, host) {
      this.subs = {};
      this.sock = net.createConnection(port, host);
      this.sock.on('connect', __bind(function() {
        this.sock.setTimeout(0);
        this.sock.setKeepAlive(true);
        return this.sock.setEncoding('utf8');
      }, this));
      this.sock.on('data', __bind(function(data) {
        var line, lines, obj, _i, _len, _results;
        lines = data.split('\r\n');
        try {
          _results = [];
          for (_i = 0, _len = lines.length; _i < _len; _i++) {
            line = lines[_i];
            if (!(line.length > 0)) {
              continue;
            }
            obj = JSON.parse(line);
            _results.push((function() {
              if (obj.pub != null) {
                this.emit('message', obj.pub.msg, obj.pub.chan);
                if (this.subs[obj.pub.chan]) {
                  return this.subs[obj.pub.chan](obj.pub.msg, obj.pub.chan);
                }
              } else {
                throw new Error('unknown message type');
              }
            }).call(this));
          }
          return _results;
        } catch (err) {
          console.error(err);
          this.emit('protocol-error', err);
          return this.sock.destroy();
        }
      }, this));
    }
    SimplePubsubClient.prototype._write = function(what) {
      try {
        this.sock.write(JSON.stringify(what) + CRLF);
        return true;
      } catch (err) {
        console.error(err);
        this.emit('io-error', err);
        this.sock.destroy();
        return false;
      }
    };
    SimplePubsubClient.prototype.subscribe = function(chan, cb) {
      if (this._write({
        sub: chan
      })) {
        return this.subs[chan] = cb;
      }
    };
    SimplePubsubClient.prototype.publish = function(msg, chan) {
      return this._write({
        pub: {
          msg: msg,
          chan: chan
        }
      });
    };
    SimplePubsubClient.prototype.close = function() {
      return this.sock.close();
    };
    return SimplePubsubClient;
  })();
  exports.connect = function(port, host) {
    if (port == null) {
      port = 9912;
    }
    if (host == null) {
      host = '127.0.0.1';
    }
    return new SimplePubsubClient(port, host);
  };
}).call(this);
